@@ -1,3 +1,24 @@
+/*
+ * This file is part of PowerDNS or dnsdist.
+ * Copyright -- PowerDNS.COM B.V. and its contributors
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of version 2 of the GNU General Public License as
+ * published by the Free Software Foundation.
+ *
+ * In addition, for the avoidance of any doubt, permission is granted to
+ * link this program with OpenSSL and to (re)distribute the binaries
+ * produced as the result of such linking.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 #include "pdns/utility.hh"
 #include <sstream>
 #include "sodbc.hh"
@@ -69,6 +90,7 @@ public:
   vector<ODBCParam> d_req_bind;
 
   SSqlStatement* bind(const string& name, ODBCParam& p) {
+    prepareStatement();
     d_req_bind.push_back(p);
     SQLRETURN result = SQLBindParameter(
       d_statement,           // StatementHandle,
@@ -88,15 +110,16 @@ public:
     return this;
   }
 
-  SSqlStatement* bind(const string& name, bool value) { return bind(name, (uint32_t)value); }
+  SSqlStatement* bind(const string& name, bool value) { prepareStatement(); return bind(name, (uint32_t)value); }
 
-  SSqlStatement* bind(const string& name, long value) { return bind(name, (unsigned long)value); }
+  SSqlStatement* bind(const string& name, long value) { prepareStatement(); return bind(name, (unsigned long)value); }
 
-  SSqlStatement* bind(const string& name, int value) { return bind(name, (uint32_t)value); }
+  SSqlStatement* bind(const string& name, int value) { prepareStatement(); return bind(name, (uint32_t)value); }
 
-  SSqlStatement* bind(const string& name, long long value) { return bind(name, (unsigned long long)value); }
+  SSqlStatement* bind(const string& name, long long value) { prepareStatement(); return bind(name, (unsigned long long)value); }
 
   SSqlStatement* bind(const string& name, uint32_t value) {
+    prepareStatement();
     ODBCParam p;
     p.ParameterValuePtr = new UDWORD {value};
     p.LenPtr = new SQLLEN {sizeof(UDWORD)};
@@ -106,6 +129,7 @@ public:
   }
 
   SSqlStatement* bind(const string& name, unsigned long value) {
+    prepareStatement();
     ODBCParam p;
     p.ParameterValuePtr = new ULONG {value};
     p.LenPtr = new SQLLEN {sizeof(ULONG)};
@@ -115,6 +139,7 @@ public:
   }
 
   SSqlStatement* bind(const string& name, unsigned long long value) {
+    prepareStatement();
     ODBCParam p;
     p.ParameterValuePtr = new unsigned long long {value};
     p.LenPtr = new SQLLEN {sizeof(unsigned long long)};
@@ -128,7 +153,7 @@ public:
     // cerr<<"asked to bind string "<<value<<endl;
 
     if(d_req_bind.size() > (d_parnum+1)) throw SSqlException("Trying to bind too many parameters.");
-
+    prepareStatement();
     ODBCParam p;
 
     p.ParameterValuePtr = (char*) new char[value.size()+1];
@@ -145,6 +170,7 @@ public:
   SSqlStatement* bindNull(const string& name) {
     if(d_req_bind.size() > (d_parnum+1)) throw SSqlException("Trying to bind too many parameters.");
 
+    prepareStatement();
     ODBCParam p;
 
     p.ParameterValuePtr = NULL;
@@ -158,6 +184,7 @@ public:
 
   SSqlStatement* execute()
   {
+    prepareStatement();
     SQLRETURN result;
     // cerr<<"execute("<<d_query<<")"<<endl;
     if (d_dolog) {
@@ -225,7 +252,7 @@ private:
      std::string errorMessage;
      if (!realTestResult(result, type, handle, message, errorMessage)) {
        releaseStatement();
-       throw errorMessage;
+       throw SSqlException(errorMessage);
      }
   }
 
@@ -455,5 +482,5 @@ void SODBC::rollback() {
 
 void SODBC::testResult(SQLRETURN result, SQLSMALLINT type, SQLHANDLE handle, const std::string & message) {
   std::string errorMessage;
-  if (!realTestResult(result, type, handle, message, errorMessage)) throw errorMessage;
+  if (!realTestResult(result, type, handle, message, errorMessage)) throw SSqlException(errorMessage);
 }
