@@ -1,12 +1,12 @@
 --
---  Table layouts for mydns 1.1.0 (Mar 2012)
---  Copyright (C) 2002-2005 Don Moore
+--  Table layouts for mydns 1.2.8.31 (Dec 2014)
+--  Copyright (C) 2002-2005 Don Moore  2007-2008 Howard Wilkinson
 --
 --  You might create these tables with a command like:
 --
 --    $ mydns --create-tables | mysql -hHOST -p -uUSER DATABASE
 --
---
+-- Originally licensed under the GNU GPLv2 or higer
 
 --
 --  Table structure for table 'soa' (zones of authority)
@@ -22,8 +22,9 @@ CREATE TABLE IF NOT EXISTS soa (
   expire     INT UNSIGNED NOT NULL default '604800',
   minimum    INT UNSIGNED NOT NULL default '86400',
   ttl        INT UNSIGNED NOT NULL default '86400',
+   active     ENUM('Y', 'N') NOT NULL DEFAULT 'Y',
   UNIQUE KEY (origin)
-);
+) Engine=MyISAM;
 
 --
 --  Table structure for table 'rr' (resource records)
@@ -31,11 +32,11 @@ CREATE TABLE IF NOT EXISTS soa (
 CREATE TABLE IF NOT EXISTS rr (
   id         INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
   zone       INT UNSIGNED NOT NULL,
-  name       CHAR(64) NOT NULL,
-  type       ENUM('A','AAAA','CNAME','HINFO','MX','NAPTR','NS','PTR','RP','SRV','TXT'),
-  data       CHAR(128) NOT NULL,
+  name       CHAR(200) NOT NULL,
+  data       VARBINARY(128) NOT NULL,
   aux        INT UNSIGNED NOT NULL,
   ttl        INT UNSIGNED NOT NULL default '86400',
-  UNIQUE KEY rr (zone,name,type,data,aux)
-);
-
+  type       ENUM('A','AAAA','CNAME','HINFO','MX','NAPTR','NS','PTR','RP','SRV','TXT'),
+  active     ENUM('Y', 'N') NOT NULL DEFAULT 'Y',
+  UNIQUE KEY rr (zone,name,type,data,aux,active)
+) Engine=MyISAM;
