@@ -1,3 +1,28 @@
+/*
+ * This file is part of PowerDNS or dnsdist.
+ * Copyright -- PowerDNS.COM B.V. and its contributors
+ * originally authored by Norbert Sendetzky
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of version 2 of the GNU General Public License as
+ * published by the Free Software Foundation.
+ *
+ * In addition, for the avoidance of any doubt, permission is granted to
+ * link this program with OpenSSL and to (re)distribute the binaries
+ * produced as the result of such linking.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include "odbxbackend.hh"
 
 
@@ -98,7 +123,7 @@ bool OdbxBackend::getDomainInfo( const string& domain, DomainInfo& di )
         	do
         	{
         		di.id = 0;
-        		di.zone = "";
+        		di.zone.clear();
         		di.masters.clear();
         		di.last_check = 0;
         		di.notified_serial = 0;
@@ -145,7 +170,7 @@ bool OdbxBackend::getDomainInfo( const string& domain, DomainInfo& di )
 
         		if( ( tmp = odbx_field_value( m_result, 1 ) ) != NULL )
         		{
-        			di.zone = string( tmp, odbx_field_length( m_result, 1 ) );
+			        di.zone = DNSName(string( tmp, odbx_field_length( m_result, 1 ) ));
         		}
 
         		if( ( tmp = odbx_field_value( m_result, 0 ) ) != NULL )
@@ -166,7 +191,7 @@ bool OdbxBackend::getDomainInfo( const string& domain, DomainInfo& di )
 
 
 
-bool OdbxBackend::getSOA( const string& domain, SOAData& sd, DNSPacket* p )
+bool OdbxBackend::getSOA( const DNSName& domain, SOAData& sd, DNSPacket* p )
 {
         const char* tmp;
 
@@ -176,7 +201,7 @@ bool OdbxBackend::getSOA( const string& domain, SOAData& sd, DNSPacket* p )
         	DLOG( L.log( m_myname + " getSOA()", Logger::Debug ) );
 
         	string stmt = getArg( "sql-lookupsoa" );
-        	string& stmtref = strbind( ":name", escape( toLower( domain ), READ ), stmt );
+        	string& stmtref = strbind( ":name", escape( domain.toStringNoDot(), READ ), stmt );
 
         	if( !execStmt( stmtref.c_str(), stmtref.size(), READ ) ) { return false; }
         	if( !getRecord( READ ) ) { return false; }
@@ -195,7 +220,7 @@ bool OdbxBackend::getSOA( const string& domain, SOAData& sd, DNSPacket* p )
         		if( ( tmp = odbx_field_value( m_result, 2 ) ) != NULL )
         		{
         			sd.ttl = strtoul( tmp, NULL, 10 );
-        		} 
+        		}
 
         		if( sd.serial == 0 && ( tmp = odbx_field_value( m_result, 1 ) ) != NULL )
         		{
@@ -209,12 +234,12 @@ bool OdbxBackend::getSOA( const string& domain, SOAData& sd, DNSPacket* p )
 
         		if( sd.nameserver.empty() )
         		{
-        			sd.nameserver = arg()["default-soa-name"];
+			        sd.nameserver = DNSName(arg()["default-soa-name"]);
         		}
 
         		if( sd.hostmaster.empty() )
         		{
-        			sd.hostmaster = "hostmaster." + domain;
+			        sd.hostmaster = DNSName("hostmaster") + DNSName(domain);
         		}
 
         		sd.db = this;
@@ -232,13 +257,13 @@ bool OdbxBackend::getSOA( const string& domain, SOAData& sd, DNSPacket* p )
 
 
 
-bool OdbxBackend::list( const string& target, int zoneid, bool include_disabled )
+bool OdbxBackend::list( const DNSName& target, int zoneid, bool include_disabled )
 {
         try
         {
         	DLOG( L.log( m_myname + " list()", Logger::Debug ) );
 
-        	m_qname = "";
+		m_qname.clear();
         	m_result = NULL;
 
         	int len = snprintf( m_buffer, sizeof( m_buffer ) - 1, "%d", zoneid );
@@ -271,7 +296,7 @@ bool OdbxBackend::list( const string& target, int zoneid, bool include_disabled
 
 
 
-void OdbxBackend::lookup( const QType& qtype, const string& qname, DNSPacket* dnspkt, int zoneid )
+void OdbxBackend::lookup( const QType& qtype, const DNSName& qname, DNSPacket* dnspkt, int zoneid )
 {
         try
         {
@@ -320,7 +345,7 @@ void OdbxBackend::lookup( const QType& qtype, const string& qname, DNSPacket* dn
         		stmtref = strbind( ":id", string( m_buffer, len ), stmtref );
         	}
 
-        	string tmp = qname;
+        	string tmp = qname.toStringNoDot();
         	stmtref = strbind( ":name", escape( toLowerByRef( tmp ), READ ), stmtref );
 
         	if( !execStmt( stmtref.c_str(), stmtref.size(), READ ) )
@@ -340,7 +365,7 @@ void OdbxBackend::lookup( const QType& qtype, const string& qname, DNSPacket* dn
 bool OdbxBackend::get( DNSResourceRecord& rr )
 {
         const char* tmp;
-
+        string priority;
 
         try
         {
@@ -348,8 +373,8 @@ bool OdbxBackend::get( DNSResourceRecord& rr )
 
         	if( getRecord( READ ) )
         	{
+
         		rr.content = "";
-        		rr.priority = 0;
         		rr.domain_id = 0;
         		rr.last_modified = 0;
         		rr.ttl = m_default_ttl;
@@ -362,7 +387,7 @@ bool OdbxBackend::get( DNSResourceRecord& rr )
 
         		if( m_qname.empty() && ( tmp = odbx_field_value( m_result, 1 ) ) != NULL )
         		{
-        			rr.qname = string( tmp, odbx_field_length( m_result, 1 ) );
+			        rr.qname = DNSName( string(tmp, odbx_field_length( m_result, 1 ) ));
         		}
 
         		if( ( tmp = odbx_field_value( m_result, 2 ) ) != NULL )
@@ -377,7 +402,7 @@ bool OdbxBackend::get( DNSResourceRecord& rr )
 
         		if( ( tmp = odbx_field_value( m_result, 4 ) ) != NULL )
         		{
-        			rr.priority = (uint16_t) strtoul( tmp, NULL, 10 );
+        			priority = string( tmp, odbx_field_length( m_result, 4 ) );
         		}
 
         		if( ( tmp = odbx_field_value( m_result, 5 ) ) != NULL )
@@ -385,6 +410,9 @@ bool OdbxBackend::get( DNSResourceRecord& rr )
         			rr.content = string( tmp, odbx_field_length( m_result, 5 ) );
         		}
 
+        		if (rr.qtype==QType::MX || rr.qtype==QType::SRV)
+        			rr.content = priority + " " + rr.content;
+
         		return true;
         	}
         }
@@ -658,10 +686,22 @@ bool OdbxBackend::feedRecord( const DNSResourceRecord& rr, string *ordername )
         		return false;
         	}
 
-        	string tmp = rr.qname;
+        	string tmp = rr.qname.toStringNoDot();
+
+        	unsigned int priority=0;
+        	string content(rr.content);
+
+        	if(rr.qtype == QType::MX || rr.qtype == QType::SRV) {
+        		priority=pdns_stou(content);
+        		string::size_type pos = content.find_first_not_of("0123456789");
+        		if(pos != string::npos)
+        			boost::erase_head(content, pos);
+        		trim_left(content);
+        	}
+
         	int len = snprintf( m_buffer, sizeof( m_buffer ) - 1, getArg( "sql-insert-record" ).c_str(), rr.domain_id,
-        		escape( toLowerByRef( tmp ), WRITE ).c_str(), rr.qtype.getName().c_str(), rr.ttl, rr.priority,
-        		escape( rr.content, WRITE ).c_str() );
+        		escape( toLowerByRef( tmp ), WRITE ).c_str(), rr.qtype.getName().c_str(), rr.ttl, priority,
+        		escape( content, WRITE ).c_str() );
 
         	if( len < 0 )
         	{
