@@ -1,5 +1,27 @@
-#include <sys/types.h>
-#include <sys/wait.h>
+/*
+ * This file is part of PowerDNS or dnsdist.
+ * Copyright -- PowerDNS.COM B.V. and its contributors
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of version 2 of the GNU General Public License as
+ * published by the Free Software Foundation.
+ *
+ * In addition, for the avoidance of any doubt, permission is granted to
+ * link this program with OpenSSL and to (re)distribute the binaries
+ * produced as the result of such linking.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include "remotebackend.hh"
 
 PipeConnector::PipeConnector(std::map<std::string,std::string> options) {
@@ -12,7 +34,7 @@ PipeConnector::PipeConnector(std::map<std::string,std::string> options) {
   d_timeout=2000;
 
   if (options.find("timeout") != options.end()) {
-     d_timeout = boost::lexical_cast<int>(options.find("timeout")->second);
+     d_timeout = std::stoi(options.find("timeout")->second);
   }
 
   d_pid = -1;
@@ -60,9 +82,9 @@ void PipeConnector::launch() {
     throw PDNSException("Unable to fork for coprocess: "+stringerror());
   else if(d_pid>0) { // parent speaking
     close(d_fd1[0]);
-    Utility::setCloseOnExec(d_fd1[1]);
+    setCloseOnExec(d_fd1[1]);
     close(d_fd2[1]);
-    Utility::setCloseOnExec(d_fd2[0]);
+    setCloseOnExec(d_fd2[0]);
     if(!(d_fp=fdopen(d_fd2[0],"r")))
       throw PDNSException("Unable to associate a file pointer with pipe: "+stringerror());
     if (d_timeout) 
@@ -92,30 +114,22 @@ void PipeConnector::launch() {
        no way to log this either - only thing we can do is make sure that our parent catches this soonest! */
   }
 
-  rapidjson::Value val;
-  rapidjson::Document init,res;
-  init.SetObject();
-  val = "initialize";
+  Json::array parameters;
+  Json msg = Json(Json::object{
+    { "method", "initialize" },
+    { "parameters", Json(options) },
+  });
 
-  init.AddMember("method",val, init.GetAllocator());
-  val.SetObject();
-  init.AddMember("parameters", val, init.GetAllocator());
-
-  for(std::map<std::string,std::string>::iterator i = options.begin(); i != options.end(); i++) {
-    val = i->second.c_str();
-    init["parameters"].AddMember(i->first.c_str(), val, init.GetAllocator());
-  }
-
-  this->send(init);
-  if (this->recv(res)==false) {
+  this->send(msg);
+  msg = nullptr;
+  if (this->recv(msg)==false) {
     L<<Logger::Error<<"Failed to initialize coprocess"<<std::endl;
   }
 }
 
-int PipeConnector::send_message(const rapidjson::Document &input)
+int PipeConnector::send_message(const Json& input)
 {
-   std::string line;
-   line = makeStringFromDocument(input);
+   auto line = input.dump();
    launch();
 
    line.append(1,'\n');
@@ -134,11 +148,10 @@ int PipeConnector::send_message(const rapidjson::Document &input)
    return sent;
 }
 
-int PipeConnector::recv_message(rapidjson::Document &output) 
+int PipeConnector::recv_message(Json& output)
 {
    std::string receive;
-   rapidjson::GenericReader<rapidjson::UTF8<> , rapidjson::MemoryPoolAllocator<> > r;
-   std::string tmp;
+   std::string err;
    std::string s_output;
    launch();
 
@@ -162,10 +175,9 @@ int PipeConnector::recv_message(rapidjson::Document &output)
        throw PDNSException("Child closed pipe");
   
       s_output.append(receive);
-      rapidjson::StringStream ss(s_output.c_str());
-      output.ParseStream<0>(ss); 
-      if (output.HasParseError() == false)
-        return s_output.size();
+      // see if it can be parsed
+      output = Json::parse(s_output, err);
+      if (output != nullptr) return s_output.size();
    }
    return 0;
 }
