@@ -1,27 +1,30 @@
 /*
-    PowerDNS Versatile Database Driven Nameserver
-    Copyright (C) 2001 - 2012  PowerDNS.COM BV
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License version 2 as 
-    published by the Free Software Foundation
-
-    Additionally, the license of this program contains a special
-    exception which allows to distribute the program in binary form when
-    it is linked against OpenSSL.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-*/
+ * This file is part of PowerDNS or dnsdist.
+ * Copyright -- PowerDNS.COM B.V. and its contributors
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of version 2 of the GNU General Public License as
+ * published by the Free Software Foundation.
+ *
+ * In addition, for the avoidance of any doubt, permission is granted to
+ * link this program with OpenSSL and to (re)distribute the binaries
+ * produced as the result of such linking.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include "dnssecinfra.hh"
 #include "namespaces.hh"
-#include <boost/foreach.hpp>
+
 #include "md5.hh"
 #include "dnsseckeeper.hh"
 #include "dns_random.hh"
@@ -32,7 +35,7 @@ extern StatBag S;
 
 /* this is where the RRSIGs begin, keys are retrieved,
    but the actual signing happens in fillOutRRSIG */
-int getRRSIGsForRRSET(DNSSECKeeper& dk, const std::string& signer, const std::string signQName, uint16_t signQType, uint32_t signTTL,
+int getRRSIGsForRRSET(DNSSECKeeper& dk, const DNSName& signer, const DNSName signQName, uint16_t signQType, uint32_t signTTL,
                      vector<shared_ptr<DNSRecordContent> >& toSign, vector<RRSIGRecordContent>& rrcs)
 {
   if(toSign.empty())
@@ -41,55 +44,33 @@ int getRRSIGsForRRSET(DNSSECKeeper& dk, const std::string& signer, const std::st
   RRSIGRecordContent rrc;
   rrc.d_type=signQType;
 
-  rrc.d_labels=countLabels(signQName); 
+  rrc.d_labels=signQName.countLabels()-signQName.isWildcard();
   rrc.d_originalttl=signTTL; 
   rrc.d_siginception=startOfWeek - 7*86400; // XXX should come from zone metadata
   rrc.d_sigexpire=startOfWeek + 14*86400;
-  rrc.d_signer = signer.empty() ? "." : toLower(signer);
+  rrc.d_signer = signer;
   rrc.d_tag = 0;
-  
-  // we sign the RRSET in toSign + the rrc w/o hash
-  
-  DNSSECKeeper::keyset_t keys = dk.getKeys(signer); // we don't want the . for the root!
-  set<int> algoHasKSK, algoHasZSK;
-  vector<DNSSECPrivateKey> signingKeys;
 
-  BOOST_FOREACH(DNSSECKeeper::keyset_t::value_type& keymeta, keys) {
-    if(keymeta.second.active) {
-      if(keymeta.second.keyOrZone)
-        algoHasKSK.insert(keymeta.first.d_algorithm);
-      else
-        algoHasZSK.insert(keymeta.first.d_algorithm);
-    }
-  }
+  DNSSECKeeper::keyset_t keys = dk.getKeys(signer);
 
-  BOOST_FOREACH(DNSSECKeeper::keyset_t::value_type& keymeta, keys) {
+  for(DNSSECKeeper::keyset_t::value_type& keymeta : keys) {
     if(!keymeta.second.active)
       continue;
 
-    if(signQType == QType::DNSKEY) {
-      // skip ZSK, if this algorithm has a KSK
-      if(!keymeta.second.keyOrZone && algoHasKSK.count(keymeta.first.d_algorithm))
-        continue;
-    } else {
-      // skip KSK, if this algorithm has a ZSK
-      if(keymeta.second.keyOrZone && algoHasZSK.count(keymeta.first.d_algorithm))
-        continue;
+    if((signQType == QType::DNSKEY && keymeta.second.keyType == DNSSECKeeper::ZSK) ||
+       (signQType != QType::DNSKEY && keymeta.second.keyType == DNSSECKeeper::KSK)) {
+      continue;
     }
 
-    signingKeys.push_back(keymeta.first);
-  }
-
-  BOOST_FOREACH(DNSSECPrivateKey& dpk, signingKeys) {
-    fillOutRRSIG(dpk, signQName, rrc, toSign);
+    fillOutRRSIG(keymeta.first, signQName, rrc, toSign);
     rrcs.push_back(rrc);
   }
   return 0;
 }
 
 // this is the entrypoint from DNSPacket
-void addSignature(DNSSECKeeper& dk, DNSBackend& db, const std::string& signer, const std::string signQName, const std::string& wildcardname, uint16_t signQType, 
-  uint32_t signTTL, DNSPacketWriter::Place signPlace, 
+void addSignature(DNSSECKeeper& dk, UeberBackend& db, const DNSName& signer, const DNSName signQName, const DNSName& wildcardname, uint16_t signQType,
+  uint32_t signTTL, DNSResourceRecord::Place signPlace,
   vector<shared_ptr<DNSRecordContent> >& toSign, vector<DNSResourceRecord>& outsigned, uint32_t origTTL)
 {
   //cerr<<"Asked to sign '"<<signQName<<"'|"<<DNSRecordContent::NumberToType(signQType)<<", "<<toSign.size()<<" records\n";
@@ -101,7 +82,7 @@ void addSignature(DNSSECKeeper& dk, DNSBackend& db, const std::string& signer, c
     dk.getPreRRSIGs(db, signer, signQName, wildcardname, QType(signQType), signPlace, outsigned, origTTL); // does it all
   }
   else {
-    if(getRRSIGsForRRSET(dk, signer, wildcardname.empty() ? signQName : wildcardname, signQType, signTTL, toSign, rrcs) < 0)  {
+    if(getRRSIGsForRRSET(dk, signer, wildcardname.countLabels() ? wildcardname : signQName, signQType, signTTL, toSign, rrcs) < 0)  {
       // cerr<<"Error signing a record!"<<endl;
       return;
     } 
@@ -114,8 +95,8 @@ void addSignature(DNSSECKeeper& dk, DNSBackend& db, const std::string& signer, c
     else
       rr.ttl=signTTL;
     rr.auth=false;
-    rr.d_place = (DNSResourceRecord::Place) signPlace;
-    BOOST_FOREACH(RRSIGRecordContent& rrc, rrcs) {
+    rr.d_place = signPlace;
+    for(RRSIGRecordContent& rrc :  rrcs) {
       rr.content = rrc.getZoneRepresentation();
       outsigned.push_back(rr);
     }
@@ -136,7 +117,7 @@ uint64_t signatureCacheSize(const std::string& str)
   return g_signatures.size();
 }
 
-void fillOutRRSIG(DNSSECPrivateKey& dpk, const std::string& signQName, RRSIGRecordContent& rrc, vector<shared_ptr<DNSRecordContent> >& toSign) 
+void fillOutRRSIG(DNSSECPrivateKey& dpk, const DNSName& signQName, RRSIGRecordContent& rrc, vector<shared_ptr<DNSRecordContent> >& toSign) 
 {
   if(!g_signatureCount)
     g_signatureCount = S.getPointer("signatures");
@@ -183,56 +164,57 @@ static bool rrsigncomp(const DNSResourceRecord& a, const DNSResourceRecord& b)
   return tie(a.d_place, a.qtype) < tie(b.d_place, b.qtype);
 }
 
-static bool getBestAuthFromSet(const set<string, CIStringCompare>& authSet, const string& name, string& auth)
+static bool getBestAuthFromSet(const set<DNSName>& authSet, const DNSName& name, DNSName& auth)
 {
-  auth.clear();
-  string sname(name);
+  auth.trimToLabels(0);
+  DNSName sname(name);
   do {
     if(authSet.find(sname) != authSet.end()) {
       auth = sname;
       return true;
     }
   }
-  while(chopOff(sname));
+  while(sname.chopOff());
   
   return false;
 }
 
-void addRRSigs(DNSSECKeeper& dk, DNSBackend& db, const set<string, CIStringCompare>& authSet, vector<DNSResourceRecord>& rrs)
+void addRRSigs(DNSSECKeeper& dk, UeberBackend& db, const set<DNSName>& authSet, vector<DNSResourceRecord>& rrs)
 {
   stable_sort(rrs.begin(), rrs.end(), rrsigncomp);
   
-  string signQName, wildcardQName;
+  DNSName signQName, wildcardQName;
   uint16_t signQType=0;
   uint32_t signTTL=0;
   uint32_t origTTL=0;
   
-  DNSPacketWriter::Place signPlace=DNSPacketWriter::ANSWER;
+  DNSResourceRecord::Place signPlace=DNSResourceRecord::ANSWER;
   vector<shared_ptr<DNSRecordContent> > toSign;
 
   vector<DNSResourceRecord> signedRecords;
-  
-  string signer;
+  signedRecords.reserve(rrs.size()*1.5);
+  //  cout<<rrs.size()<<", "<<sizeof(DNSResourceRecord)<<endl;
+  DNSName signer;
   for(vector<DNSResourceRecord>::const_iterator pos = rrs.begin(); pos != rrs.end(); ++pos) {
     if(pos != rrs.begin() && (signQType != pos->qtype.getCode()  || signQName != pos->qname)) {
       if(getBestAuthFromSet(authSet, signQName, signer))
         addSignature(dk, db, signer, signQName, wildcardQName, signQType, signTTL, signPlace, toSign, signedRecords, origTTL);
     }
     signedRecords.push_back(*pos);
-    signQName= pos->qname;
-    wildcardQName = pos->wildcardname;
+    signQName= pos->qname.makeLowerCase();
+    if(!pos->wildcardname.empty())
+      wildcardQName = pos->wildcardname.makeLowerCase();
+    else
+      wildcardQName.clear();
     signQType = pos ->qtype.getCode();
     if(pos->signttl)
       signTTL = pos->signttl;
     else
       signTTL = pos->ttl;
     origTTL = pos->ttl;
-    signPlace = (DNSPacketWriter::Place) pos->d_place;
+    signPlace = pos->d_place;
     if(pos->auth || pos->qtype.getCode() == QType::DS) {
       string content = pos->content;
-      if(pos->qtype.getCode()==QType::MX || pos->qtype.getCode() == QType::SRV) {  
-        content = lexical_cast<string>(pos->priority) + " " + pos->content;
-      }
       if(!pos->content.empty() && pos->qtype.getCode()==QType::TXT && pos->content[0]!='"') {
         content="\""+pos->content+"\"";
       }
