@@ -1,24 +1,24 @@
 /*
-    PowerDNS Versatile Database Driven Nameserver
-    Copyright (C) 2002-2014  PowerDNS.COM BV
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License version 2 as 
-    published by the Free Software Foundation
-
-    Additionally, the license of this program contains a special
-    exception which allows to distribute the program in binary form when
-    it is linked against OpenSSL.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-*/
+ * This file is part of PowerDNS or dnsdist.
+ * Copyright -- PowerDNS.COM B.V. and its contributors
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of version 2 of the GNU General Public License as
+ * published by the Free Software Foundation.
+ *
+ * In addition, for the avoidance of any doubt, permission is granted to
+ * link this program with OpenSSL and to (re)distribute the binaries
+ * produced as the result of such linking.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -325,7 +325,7 @@ vector<DNSResourceRecord> PacketHandler::getBestDNAMESynth(DNSPacket *p, SOAData
     if(!ret.empty())
       return ret;
     if(subdomain.countLabels())
-      prefix+= DNSName(subdomain.getRawLabels()[0]); // XXX DNSName pain this feels wrong
+      prefix.appendRawLabel(subdomain.getRawLabels()[0]);
     if(subdomain == sd.qname) // stop at SOA
       break;
 
@@ -405,31 +405,30 @@ int PacketHandler::doAdditionalProcessingAndDropAA(DNSPacket *p, DNSPacket *r, c
         trim_left(content);
       }
 
-      QType qtypes[2];
-      qtypes[0]="A"; qtypes[1]="AAAA";
-      for(int n=0 ; n < d_doIPv6AdditionalProcessing + 1; ++n) {
-        if (i->qtype.getCode()==QType::SRV) {
-          vector<string>parts;
-          stringtok(parts, content);
-          if (parts.size() >= 3) {
-            B.lookup(qtypes[n], DNSName(parts[2]), p);
-          }
-          else
-            continue;
+      if (i->qtype.getCode()==QType::SRV) {
+        vector<string>parts;
+        stringtok(parts, content);
+        if (parts.size() >= 3) {
+          B.lookup(QType(d_doIPv6AdditionalProcessing ? QType::ANY : QType::A), DNSName(parts[2]), p);
         }
-        else {
-          B.lookup(qtypes[n], DNSName(content), p);
-        }
-        while(B.get(rr)) {
-          if(rr.domain_id!=i->domain_id && ::arg()["out-of-zone-additional-processing"]=="no") {
-            DLOG(L<<Logger::Warning<<"Not including out-of-zone additional processing of "<<i->qname<<" ("<<rr.qname<<")"<<endl);
-            continue; // not adding out-of-zone additional data
-          }
-          if(rr.auth && !rr.qname.isPartOf(soadata.qname)) // don't sign out of zone data using the main key 
-            rr.auth=false;
-          rr.d_place=DNSResourceRecord::ADDITIONAL;
-          r->addRecord(rr);
+        else
+          continue;
+      }
+      else {
+        B.lookup(QType(d_doIPv6AdditionalProcessing ? QType::ANY : QType::A), DNSName(content), p);
+      }
+      while(B.get(rr)) {
+        if(rr.qtype.getCode() != QType::A && rr.qtype.getCode()!=QType::AAAA)
+          continue;
+        if(rr.domain_id!=i->domain_id && ::arg()["out-of-zone-additional-processing"]=="no") {
+          DLOG(L<<Logger::Warning<<"Not including out-of-zone additional processing of "<<i->qname<<" ("<<rr.qname<<")"<<endl);
+          continue; // not adding out-of-zone additional data
         }
+        
+        if(rr.auth && !rr.qname.isPartOf(soadata.qname)) // don't sign out of zone data using the main key 
+          rr.auth=false;
+        rr.d_place=DNSResourceRecord::ADDITIONAL;
+        r->addRecord(rr);
       }
     }
   }
