@@ -1,7 +1,10 @@
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include "signingpipe.hh"
 #include "misc.hh"
 #include <poll.h>
-#include <boost/foreach.hpp>
+
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
@@ -63,13 +66,13 @@ try
   return 0;
 }
 catch(...) {
-  L<<Logger::Error<<"unknown exception in signing thread occurred"<<endl;
+  L<<Logger::Error<<"Unknown exception in signing thread occurred"<<endl;
   return 0;
 }
 
-ChunkedSigningPipe::ChunkedSigningPipe(const std::string& signerName, bool mustSign, const pdns::string& servers, unsigned int workers) 
-  : d_queued(0), d_outstanding(0), d_signer(signerName), d_maxchunkrecords(100), d_numworkers(workers), d_tids(d_numworkers),
-    d_mustSign(mustSign), d_final(false), d_submitted(0)
+ChunkedSigningPipe::ChunkedSigningPipe(const DNSName& signerName, bool mustSign, const string& servers, unsigned int workers)
+  : d_signed(0), d_queued(0), d_outstanding(0), d_numworkers(workers), d_submitted(0), d_signer(signerName),
+    d_maxchunkrecords(100), d_tids(d_numworkers), d_mustSign(mustSign), d_final(false)
 {
   d_rrsetToSign = new rrset_t;
   d_chunks.push_back(vector<DNSResourceRecord>()); // load an empty chunk
@@ -82,10 +85,10 @@ ChunkedSigningPipe::ChunkedSigningPipe(const std::string& signerName, bool mustS
   for(unsigned int n=0; n < d_numworkers; ++n) {
     if(socketpair(AF_UNIX, SOCK_STREAM, 0, fds) < 0) 
       throw runtime_error("Unable to create communication socket in for ChunkedSigningPipe");
-    Utility::setCloseOnExec(fds[0]);
-    Utility::setCloseOnExec(fds[1]);
+    setCloseOnExec(fds[0]);
+    setCloseOnExec(fds[1]);
     pthread_create(&d_tids[n], 0, helperWorker, (void*) new StartHelperStruct(this, n, fds[1]));
-    Utility::setNonBlocking(fds[0]);
+    setNonBlocking(fds[0]);
     d_sockets.push_back(fds[0]);
   }
 }
@@ -95,35 +98,27 @@ ChunkedSigningPipe::~ChunkedSigningPipe()
   delete d_rrsetToSign;
   if(!d_mustSign)
     return;
-  BOOST_FOREACH(int fd, d_sockets) {
+  for(int fd :  d_sockets) {
     close(fd); // this will trigger all threads to exit
   }
     
   void* res;
-  BOOST_FOREACH(pthread_t& tid, d_tids) {
+  for(pthread_t& tid :  d_tids) {
     pthread_join(tid, &res);
   }
   //cout<<"Did: "<<d_signed<<", records (!= chunks) submitted: "<<d_submitted<<endl;
 }
 
 namespace {
-bool dedupLessThan(const DNSResourceRecord& a, const DNSResourceRecord &b)
+bool
+dedupLessThan(const DNSResourceRecord& a, const DNSResourceRecord &b)
 {
-  uint16_t aprio = 0, bprio = 0;
-  if (a.qtype.getCode() == QType::MX || a.qtype.getCode() == QType::SRV)
-    aprio = a.priority;
-  if (b.qtype.getCode() == QType::MX || b.qtype.getCode() == QType::SRV)
-    bprio = b.priority;
-  return tie(a.content, aprio) < tie(b.content, bprio);
+  return (tie(a.content, a.ttl) < tie(b.content, b.ttl));
 }
 
 bool dedupEqual(const DNSResourceRecord& a, const DNSResourceRecord &b)
 {
-  if(a.content != b.content)
-    return false;
-  if(a.qtype.getCode() == QType::MX || a.qtype.getCode() == QType::SRV)
-    return a.priority == b.priority;
-  return true;
+  return(tie(a.content, a.ttl) == tie(b.content, b.ttl));
 }
 }
 
@@ -138,7 +133,7 @@ bool ChunkedSigningPipe::submit(const DNSResourceRecord& rr)
 {
   ++d_submitted;
   // check if we have a full RRSET to sign
-  if(!d_rrsetToSign->empty() && (d_rrsetToSign->begin()->qtype.getCode() != rr.qtype.getCode()  ||  !pdns_iequals(d_rrsetToSign->begin()->qname, rr.qname))) 
+  if(!d_rrsetToSign->empty() && (d_rrsetToSign->begin()->qtype.getCode() != rr.qtype.getCode()  ||  d_rrsetToSign->begin()->qname != rr.qname)) 
   {
     dedupRRSet();
     sendRRSetToWorker();
@@ -166,7 +161,7 @@ pair<vector<int>, vector<int> > ChunkedSigningPipe::waitForRW(bool rd, bool wr,
   
   int res = poll(&pfds[0], pfds.size(), (seconds < 0) ? -1 : (seconds * 1000)); // -1 = infinite
   if(res < 0)
-    unixDie("polling for activity from signers, "+lexical_cast<string>(d_sockets.size()));
+    unixDie("polling for activity from signers, "+std::to_string(d_sockets.size()));
   pair<vector<int>, vector<int> > vects;
   for(unsigned int n = 0; n < pfds.size(); ++n) 
     if(pfds[n].revents & POLLIN)
@@ -229,7 +224,7 @@ void ChunkedSigningPipe::sendRRSetToWorker() // it sounds so socialist!
     while(d_outstanding) {
       chunk_t* chunk;
       
-      BOOST_FOREACH(int fd, rwVect.first) {
+      for(int fd :  rwVect.first) {
         if(d_eof.count(fd))
           continue;
         
@@ -273,7 +268,7 @@ void ChunkedSigningPipe::sendRRSetToWorker() // it sounds so socialist!
 unsigned int ChunkedSigningPipe::getReady()
 {
    unsigned int sum=0; 
-   BOOST_FOREACH(const std::vector<DNSResourceRecord>& v, d_chunks) {
+   for(const std::vector<DNSResourceRecord>& v :  d_chunks) {
      sum += v.size(); 
    }
    return sum;
@@ -292,7 +287,7 @@ try
       break;
     if(res < 0)
       unixDie("reading object pointer to sign from pdns");
-    set<string, CIStringCompare> authSet;
+    set<DNSName> authSet;
     authSet.insert(d_signer);
     addRRSigs(dk, db, authSet, *chunk);
     ++d_signed;
@@ -325,7 +320,7 @@ vector<DNSResourceRecord> ChunkedSigningPipe::getChunk(bool final)
     d_final = true;
     flushToSign();
     
-    BOOST_FOREACH(int fd, d_sockets) {
+    for(int fd :  d_sockets) {
       shutdown(fd, SHUT_WR); // perhaps this transmits EOF the other side
       //cerr<<"shutdown of "<<fd<<endl;
     }
@@ -364,7 +359,7 @@ vector<DNSResourceRecord> ChunkedSigningPipe::getChunk(bool final)
       signal(SIGCHLD, SIG_IGN);
       if(!fork()) { // child
         dup2(fds[1], 0);
-        execl("./pdnssec", "./pdnssec", "--config-dir=./", "signing-slave", NULL);
+        execl("./pdnsutil", "./pdnsutil", "--config-dir=./", "signing-slave", NULL);
         // helperWorker(new StartHelperStruct(this, n));
         return;
       }
