@@ -1,12 +1,16 @@
 #define BOOST_TEST_DYN_LINK
 #define BOOST_TEST_NO_MAIN
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <boost/test/unit_test.hpp>
 #include <boost/assign/list_of.hpp>
-#include <boost/foreach.hpp>
+
 #include <boost/tuple/tuple.hpp>
 #include "misc.hh"
 #include "dns.hh"
+#include <arpa/inet.h>
 #include <utility>
 
 using std::string;
@@ -67,7 +71,7 @@ BOOST_AUTO_TEST_CASE(test_pdns_ilexicographical_compare) {
     (case_t(std::string("Abc"), std::string("abc"), false))
   ;
 
-  BOOST_FOREACH(const case_t& val, cases) {
+  for(const case_t& val :  cases) {
     bool res;
     res = pdns_ilexicographical_compare(val.get<0>(), val.get<1>());
     BOOST_CHECK_EQUAL(res, val.get<2>());
@@ -90,7 +94,7 @@ BOOST_AUTO_TEST_CASE(test_pdns_iequals) {
     (case_t(std::string("Abc"), std::string("abc"), true))
   ;
 
-  BOOST_FOREACH(const case_t& val, cases) {
+  for(const case_t& val :  cases) {
     bool res;
     res = pdns_iequals(val.get<0>(), val.get<1>());
     BOOST_CHECK_EQUAL(res, val.get<2>());
@@ -114,7 +118,7 @@ BOOST_AUTO_TEST_CASE(test_makeRelative) {
 }
 
 BOOST_AUTO_TEST_CASE(test_AtomicCounter) {
-    AtomicCounter ac;
+    AtomicCounter ac(0);
     ++ac;
     ++ac;
     BOOST_CHECK_EQUAL(ac, 2);
@@ -141,5 +145,40 @@ BOOST_AUTO_TEST_CASE(test_parseService) {
     BOOST_CHECK_EQUAL(tp.port, 25);
 }
 
+BOOST_AUTO_TEST_CASE(test_ternary) {
+  int maxqps=1024;
+  BOOST_CHECK_EQUAL(defTer(maxqps, 16384), maxqps);
+  BOOST_CHECK_EQUAL(defTer(0, 16384), 16384);
+
+  int* qps=0;
+  BOOST_CHECK_EQUAL(*defTer(qps, &maxqps), 1024);
+}
+
+BOOST_AUTO_TEST_CASE(test_SimpleMatch) {
+  BOOST_CHECK_EQUAL(SimpleMatch("").match(std::string("")), true);
+  BOOST_CHECK_EQUAL(SimpleMatch("?").match(std::string("")), false);
+  BOOST_CHECK_EQUAL(SimpleMatch("*").match(std::string("")), true);
+
+  BOOST_CHECK_EQUAL(SimpleMatch("abc").match(std::string("abc")), true);
+  BOOST_CHECK_EQUAL(SimpleMatch("abc").match(std::string("ab")), false);
+  BOOST_CHECK_EQUAL(SimpleMatch("abc").match(std::string("bc")), false);
+
+  BOOST_CHECK_EQUAL(SimpleMatch("?").match(std::string("a")), true);
+  BOOST_CHECK_EQUAL(SimpleMatch("a?c").match(std::string("abc")), true);
+  BOOST_CHECK_EQUAL(SimpleMatch("a?c").match(std::string("ab")), false);
+  BOOST_CHECK_EQUAL(SimpleMatch("a?c").match(std::string("bc")), false);
+
+  BOOST_CHECK_EQUAL(SimpleMatch("*").match(std::string("*")), true);
+  BOOST_CHECK_EQUAL(SimpleMatch("a*c").match(std::string("abc")), true);
+  BOOST_CHECK_EQUAL(SimpleMatch("a*c").match(std::string("ab")), false);
+  BOOST_CHECK_EQUAL(SimpleMatch("a*c").match(std::string("bc")), false);
+
+  BOOST_CHECK_EQUAL(SimpleMatch("*").match(std::string("abcdefghj")), true);
+  BOOST_CHECK_EQUAL(SimpleMatch("*a").match(std::string("abca")), true);
+  BOOST_CHECK_EQUAL(SimpleMatch("*a").match(std::string("abcb")), false);
+  BOOST_CHECK_EQUAL(SimpleMatch("abc*").match(std::string("abcabcabcabacabac")), true);
+  BOOST_CHECK_EQUAL(SimpleMatch("abc*").match(std::string("abc")), true);
+}
+
 BOOST_AUTO_TEST_SUITE_END()
 
