@@ -1,3 +1,6 @@
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include "dnsparser.hh"
 #include "dnswriter.hh"
 #include "sstuff.hh"
@@ -36,17 +39,17 @@ try
 
   vector<uint8_t> packet;
   
-  DNSPacketWriter pw(packet, argv[3], DNSRecordContent::TypeToNumber(argv[4]));
+  DNSPacketWriter pw(packet, DNSName(argv[3]), DNSRecordContent::TypeToNumber(argv[4]));
 
   pw.getHeader()->id=htons(0x4831);
   
   string key;
   B64Decode("Syq9L9WrBWdxBC+HxKok2g==", key);
 
-  string keyname("pdns-b-aa");
+  DNSName keyname("pdns-b-aa");
 
   TSIGRecordContent trc;
-  trc.d_algoName="hmac-md5.sig-alg.reg.int.";
+  trc.d_algoName=DNSName("hmac-md5.sig-alg.reg.int");
   trc.d_time=time(0);
   trc.d_fudge=300;
   trc.d_origID=ntohs(pw.getHeader()->id);
@@ -62,7 +65,7 @@ try
   string reply;
   sock.recvFrom(reply, dest);
 
-  MOADNSParser mdp(reply);
+  MOADNSParser mdp(false, reply);
   cout<<"Reply to question for qname='"<<mdp.d_qname<<"', qtype="<<DNSRecordContent::NumberToType(mdp.d_qtype)<<endl;
   cout<<"Rcode: "<<mdp.d_header.rcode<<", RD: "<<mdp.d_header.rd<<", QR: "<<mdp.d_header.qr;
   cout<<", TC: "<<mdp.d_header.tc<<", AA: "<<mdp.d_header.aa<<", opcode: "<<mdp.d_header.opcode<<endl;
@@ -74,7 +77,7 @@ try
     cout<<"\t"<<i->first.d_ttl<<"\t"<< i->first.d_content->getZoneRepresentation()<<"\n";
     
     if(i->first.d_type == QType::TSIG)
-      trc2 = boost::dynamic_pointer_cast<TSIGRecordContent>(i->first.d_content);
+      trc2 = std::dynamic_pointer_cast<TSIGRecordContent>(i->first.d_content);
   }
 
   if(mdp.getTSIGPos()) {    
@@ -89,8 +92,12 @@ try
   }
 #endif
   seedRandom("/dev/urandom");
-  cerr<<"Keyname: '"<<keyname<<"', algo: '"<<trc.d_algoName<<"', key: '"<<Base64Encode(key)<<"'\n";
-  AXFRRetriever axfr(dest, "b.aa", keyname, "hmac-md5", key);
+  cerr<<"Keyname: '"<<keyname.toString()<<"', algo: '"<<trc.d_algoName.toString()<<"', key: '"<<Base64Encode(key)<<"'\n";
+  TSIGTriplet tt;
+  tt.name=keyname;
+  tt.algo=DNSName("hmac-md5");
+  tt.secret=key;
+  AXFRRetriever axfr(dest, DNSName("b.aa"), tt);
   vector<DNSResourceRecord> res;
   while(axfr.getChunk(res)) {
   }
