@@ -21,6 +21,9 @@
 */
 /* accepts a named.conf or a zone as parameter and outputs heaps of sql */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <unistd.h>
 #include <string>
 #include <map>
@@ -41,40 +44,22 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <boost/foreach.hpp>
 
+#include "json11.hpp"
+
+using namespace json11;
 
 StatBag S;
 static int g_numRecords;
 
-static void quoteValue(string &value) 
-{
-  string tmp;
-  size_t opos,pos;
-
-  // no point doing it if there isn't anything to do
-  if (value.find_first_of("\\\\\"") == string::npos) return;
-
-  pos = opos = 0;
-  while((pos = value.find_first_of("\\\\\"", opos)) != string::npos) 
-  {
-     tmp += value.substr(opos, pos - opos);
-     tmp += "\\";
-     tmp += value[pos];
-     opos = pos+1;
-  }
-
-  value = tmp;
-}
-
-
-static string emitRecord(const string& zoneName, const string &qname, const string &qtype, const string &ocontent, int ttl, int prio)
+static Json::object emitRecord(const string& zoneName, const DNSName &DNSqname, const string &qtype, const string &ocontent, int ttl)
 {
+  int prio=0;
   string retval;
   g_numRecords++;
   string content(ocontent);
   if(qtype == "MX" || qtype == "SRV") { 
-    prio=atoi(content.c_str());
+    prio=pdns_stou(content);
     
     string::size_type pos = content.find_first_not_of("0123456789");
     if(pos != string::npos)
@@ -82,35 +67,15 @@ static string emitRecord(const string& zoneName, const string &qname, const stri
     trim_left(content);
   }
 
-  quoteValue(content);
+  Json::object dict;
  
-  retval = "{";
-  retval += "\"name\":\"";
-  retval += qname;
-  retval += "\",";
-  retval += "\"type\":\"";
-  retval += qtype;
-  retval += "\",";
-  retval += "\"ttl\":";
-  retval += lexical_cast<string>(ttl);
-  retval += ",";
-  retval += "\"prio\":";
-  retval += lexical_cast<string>(prio);
-  retval += ",";
-  retval += "\"content\":\"";
-  retval += content;
-  retval += "\"}";
- 
-  return retval;
-}
+  dict["name"] = DNSqname.toString();
+  dict["type"] = qtype;
+  dict["ttl"] = ttl;
+  dict["prio"] = prio;
+  dict["content"] = content;
 
-static void emitJson(vector<string> &data)
-{
-   size_t l = data.size();
-   cout << "[";
-   for(size_t i=0;i<l-1;i++) 
-      cout << data[i] << ",";
-   cout << data[l-1] << "]";
+  return dict;
 }
 
 /* 2 modes of operation, either --named or --zone (the latter needs $ORIGIN) 
@@ -146,6 +111,7 @@ try
     ::arg().set("soa-expire-default","Do not change")="0";
 
     ::arg().setCmd("help","Provide a helpful message");
+    ::arg().setCmd("version","Print the version");
 
     S.declare("logmessages");
 
@@ -153,7 +119,12 @@ try
     string zonefile="";
 
     ::arg().parse(argc, argv);
-  
+
+    if(::arg().mustDo("version")){
+      cerr<<"zone2json "<<VERSION<<endl;
+      exit(0);
+    }
+
     if(::arg().mustDo("help")) {
       cout<<"syntax:"<<endl<<endl;
       cout<<::arg().helpstring()<<endl;
@@ -189,8 +160,8 @@ try
 
       int numdomains=domains.size();
       int tick=numdomains/100;
-      cout <<"[";
-   
+      cout << "[";
+
       for(vector<BindDomainInfo>::const_iterator i=domains.begin();
           i!=domains.end();
           ++i)
@@ -201,15 +172,20 @@ try
           }
           lines.clear(); 
           try {
+            Json::object obj;
+            Json::array recs;
             ZoneParserTNG zpt(i->filename, i->name, BP.getDirectory());
             DNSResourceRecord rr;
+            obj["name"] = i->name.toString();
+
             while(zpt.get(rr)) 
-              lines.push_back(emitRecord(i->name, rr.qname, rr.qtype.getName(), rr.content, rr.ttl, rr.priority));
-            cout << "{\"name\":\"" << i->name << "\",\"records\": ";
-            emitJson(lines);
-            cout << "},";
+              recs.push_back(emitRecord(i->name.toString(), rr.qname, rr.qtype.getName(), rr.content, rr.ttl));
+            obj["records"] = recs;
+            Json tmp = obj;
+            cout<<tmp.dump();
+            if(i+1 < domains.end()) cout<<",";
             num_domainsdone++;
-          } 
+          }
           catch(std::exception &ae) {
             if(!::arg().mustDo("on-error-resume-next"))
               throw;
@@ -225,18 +201,26 @@ try
           if(!tick || !((count++)%tick))
             cerr<<"\r"<<count*100/numdomains<<"% done ("<<i->filename<<")\033\133\113";
         }
-      cout << "]\n";
+      cout << "]" << endl;
       cerr<<"\r100% done\033\133\113"<<endl;
     }
     else {
-      ZoneParserTNG zpt(zonefile, ::arg()["zone-name"]);
+      ZoneParserTNG zpt(zonefile, DNSName(::arg()["zone-name"]));
       DNSResourceRecord rr;
-      string zname; 
-      cout << "{\"name\":\"" << ::arg()["zone-name"] << "\",\"records\":";
+      string zname;
+      Json::object obj;
+      Json::array records;
+
+      obj["name"] = ::arg()["zone-name"];
+
       while(zpt.get(rr)) 
-        lines.push_back(emitRecord(::arg()["zone-name"], rr.qname, rr.qtype.getName(), rr.content, rr.ttl, rr.priority));
-      emitJson(lines);
-      cout << "}\n";
+        records.push_back(emitRecord(::arg()["zone-name"], rr.qname, rr.qtype.getName(), rr.content, rr.ttl));
+      obj["records"] = records;
+
+      Json tmp = obj;
+
+      cout<<tmp.dump()<<endl;
+
       num_domainsdone=1;
     }
     cerr<<num_domainsdone<<" domains were fully parsed, containing "<<g_numRecords<<" records\n";
